using System;
using System.Collections.Generic;
using System.Web.UI;
using Telerik.WebControls;

public partial class DefaultCS : Page 
{
    private Stack<string> DragAndDropStack
    {
        get
        {
            Stack<string> stack = (Stack<string>)Session["DragAndDropStack"];

            if (stack == null)
            {
                stack = new Stack<string>();

                Session["DragAndDropStack"] = stack;
            }

            return stack;
        }
    }
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.IsPostBack)
        {
            DragAndDropStack.Clear();

            this.RadTreeView1.LoadContentFile("~/tree.xml");
        }
    }

    protected void RadTreeView1_NodeDrop(object o, RadTreeNodeEventArgs e)
    {
        DragAndDropStack.Push(this.RadTreeView1.GetXml());
        this.Button1.Enabled = true;

        RadTreeNode sourceNode = e.SourceDragNode;
        RadTreeNode destNode = e.DestDragNode;
        RadTreeViewDropPosition dropPosition = e.DropPosition;

        if (destNode != null)
        {
            if (sourceNode.TreeView.SelectedNodes.Count <= 1)
            {
                PerformDragAndDrop(dropPosition, sourceNode, destNode);
            }
            else if (sourceNode.TreeView.SelectedNodes.Count > 1)
            {
                foreach (RadTreeNode node in sourceNode.TreeView.SelectedNodes)
                {
                    PerformDragAndDrop(dropPosition, node, destNode);
                }
            }

            destNode.Expanded = true;

            sourceNode.TreeView.ClearSelectedNodes();
        }
    }

    private static void PerformDragAndDrop(RadTreeViewDropPosition dropPosition, RadTreeNode sourceNode, RadTreeNode destNode)
    {
        if (sourceNode.Equals(destNode) || sourceNode.IsAncestorOf(destNode))
        {
            return;
        }

        sourceNode.Owner.Nodes.Remove(sourceNode);

        switch (dropPosition)
        {
            case RadTreeViewDropPosition.Over:
                // child
                if (!sourceNode.IsAncestorOf(destNode))
                {
                    destNode.Nodes.Add(sourceNode);
                }
                break;

            case RadTreeViewDropPosition.Above:
                // sibling - above                    
                destNode.InsertBefore(sourceNode);
                break;

            case RadTreeViewDropPosition.Below:
                // sibling - below
                destNode.InsertAfter(sourceNode);
                break;
        }
    }

    protected void Button1_Click(object sender, EventArgs e)
    {
        string xmlString = DragAndDropStack.Pop();

        this.RadTreeView1.LoadXmlString(xmlString);

        if (DragAndDropStack.Count == 0)
        {
            this.Button1.Enabled = false;
        }
    }
}